import 'package:flutter/material.dart';
import 'package:dotted_border/dotted_border.dart';
import 'package:senior_design_system/senior_design_system.dart';
import 'package:senior_design_tokens/senior_design_tokens.dart';

enum SeniorSquareButtonsMenuItemType {
  primary,
  secondary,
  ghost,
}

class SeniorSquareButtonsMenuItemData {
  final Key? key;
  final IconData icon;
  final String text;
  final SeniorSquareButtonsMenuItemType type;
  final Function() onTap;
  final bool disabled;

  late final bool withCheckbox;
  late final bool isChecked;
  late final Function(bool?) onCheck;

  SeniorSquareButtonsMenuItemData({
    required this.icon,
    required this.text,
    required this.type,
    required this.onTap,
    this.disabled = false,
    this.key,
  }) {
    this.withCheckbox = false;
    this.isChecked = false;
    this.onCheck = (_) {};
  }

  SeniorSquareButtonsMenuItemData.checkbox({
    required this.icon,
    required this.text,
    required this.type,
    required this.onTap,
    required this.onCheck,
    required this.isChecked,
    this.disabled = false,
    this.key,
  }) {
    this.withCheckbox = true;
  }
}

class SeniorSquareButtonsMenu extends StatelessWidget {
  final List<SeniorSquareButtonsMenuItemData> items;

  final EdgeInsetsGeometry padding;

  const SeniorSquareButtonsMenu({
    required this.items,
    this.padding = const EdgeInsets.all(4.0),
    Key? key,
  }) : super(key: key);

  factory SeniorSquareButtonsMenu.grid({
    Key? key,
    required List<SeniorSquareButtonsMenuItemData> items,
    int crossAxisCount,
  }) = _SeniorSquadButtonsMenuGrid;

  Color _getBackgroundColor({
    required SeniorSquareButtonsMenuItemType type,
    bool disabled = false,
  }) {
    switch (type) {
      case SeniorSquareButtonsMenuItemType.primary:
        return SeniorColors.primaryColor.withOpacity(
          disabled ? 0.3 : 1.0,
        );
      case SeniorSquareButtonsMenuItemType.secondary:
        return SeniorColors.secondaryColor600.withOpacity(
          disabled ? 0.3 : 1.0,
        );
      case SeniorSquareButtonsMenuItemType.ghost:
        return Colors.transparent;
    }
  }

  Color _getDrawingColor(SeniorSquareButtonsMenuItemType type) {
    switch (type) {
      case SeniorSquareButtonsMenuItemType.ghost:
        return SeniorColors.secondaryColor600;
      default:
        return SeniorColors.pureWhite;
    }
  }

  Widget buildMenuItem({
    required SeniorSquareButtonsMenuItemData item,
    required double itemsSize,
  }) {
    final double iconSize = itemsSize * 0.3;
    final bool smallSize = itemsSize < 100.0;

    return Padding(
      key: item.key,
      padding: padding,
      child: Tooltip(
        message: item.text,
        child: Material(
          color: Colors.transparent,
          child: InkWell(
            borderRadius: BorderRadius.circular(SeniorRadius.xbig),
            onTap: item.onTap,
            child: DottedBorder(
              borderType: BorderType.RRect,
              radius: Radius.circular(SeniorRadius.xbig),
              padding: const EdgeInsets.all(0),
              color: item.type == SeniorSquareButtonsMenuItemType.ghost
                  ? SeniorColors.secondaryColor600
                  : Colors.transparent,
              child: Ink(
                height: itemsSize,
                width: itemsSize,
                padding: const EdgeInsets.all(8.0),
                decoration: BoxDecoration(
                  borderRadius: BorderRadius.circular(SeniorRadius.xbig),
                  gradient: item.type == SeniorSquareButtonsMenuItemType.primary
                      ? LinearGradient(
                          begin: Alignment.bottomLeft,
                          end: Alignment.topRight,
                          colors: SeniorColors.primaryGradientColors,
                        )
                      : null,
                  color: _getBackgroundColor(
                    type: item.type,
                    disabled: item.disabled,
                  ),
                ),
                child: Stack(
                  alignment: Alignment.center,
                  children: [
                    Visibility(
                      visible: item.withCheckbox && !smallSize,
                      child: Align(
                        alignment: Alignment.topLeft,
                        child: Container(
                          decoration: BoxDecoration(
                            color: SeniorColors.pureWhite,
                            borderRadius:
                                new BorderRadius.circular(SeniorRadius.small),
                          ),
                          child: SeniorCheckbox(
                            value: item.isChecked,
                            onChanged: item.onCheck,
                          ),
                        ),
                      ),
                    ),
                    Column(
                      mainAxisAlignment: !smallSize
                          ? MainAxisAlignment.center
                          : MainAxisAlignment.center,
                      crossAxisAlignment: CrossAxisAlignment.center,
                      children: [
                        Icon(
                          item.icon,
                          color: _getDrawingColor(item.type),
                          size: iconSize,
                        ),
                        Visibility(
                          visible: !smallSize,
                          child: Padding(
                            padding: EdgeInsets.only(
                              top: itemsSize * 0.12,
                            ),
                            child: Text(
                              item.text,
                              textAlign: TextAlign.center,
                              overflow: TextOverflow.ellipsis,
                              maxLines: 2,
                              style: SeniorTypography.small(
                                color: _getDrawingColor(item.type),
                              ),
                            ),
                          ),
                        ),
                      ],
                    ),
                  ],
                ),
              ),
            ),
          ),
        ),
      ),
    );
  }

  @override
  Widget build(BuildContext context) {
    return SingleChildScrollView(
      scrollDirection: Axis.horizontal,
      child: Row(
        children: items
            .map((item) => buildMenuItem(item: item, itemsSize: 100.0))
            .toList(),
      ),
    );
  }
}

class _SeniorSquadButtonsMenuGrid extends SeniorSquareButtonsMenu {
  late final int crossAxisCount;

  _SeniorSquadButtonsMenuGrid({
    Key? key,
    required List<SeniorSquareButtonsMenuItemData> items,
    int crossAxisCount = 3,
  }) : super(key: key, items: items) {
    this.crossAxisCount = crossAxisCount <= 6 ? crossAxisCount : 6;
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      child: LayoutBuilder(
        builder: (BuildContext context, BoxConstraints constraints) {
          double menuWidth = constraints.constrainWidth();
          int slots = items.length;
          while (slots % crossAxisCount != 0) slots++;
          double itemsSize = (menuWidth - 8 * crossAxisCount) / crossAxisCount;
          int linesCount = (items.length / crossAxisCount).ceil();
          double menuHeight =
              slots / crossAxisCount * itemsSize + linesCount * 8;

          return SizedBox(
            height: menuHeight,
            width: menuWidth,
            child: GridView.count(
              crossAxisCount: crossAxisCount,
              physics: RangeMaintainingScrollPhysics(),
              children: items
                  .map((item) => Container(
                        height: itemsSize,
                        width: itemsSize,
                        child: buildMenuItem(item: item, itemsSize: itemsSize),
                      ))
                  .toList(),
            ),
          );
        },
      ),
    );
  }
}
